
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY part3 IS
   PORT ( CLOCK_50, CLOCK2_50, AUD_DACLRCK   : IN    STD_LOGIC;
          AUD_ADCLRCK, AUD_BCLK, AUD_ADCDAT  : IN    STD_LOGIC;
          KEY                                : IN    STD_LOGIC_VECTOR(0 DOWNTO 0);
          FPGA_I2C_SDAT                      : INOUT STD_LOGIC;
          FPGA_I2C_SCLK, AUD_DACDAT, AUD_XCK : OUT   STD_LOGIC );
END part3;

ARCHITECTURE Behavior OF part3 IS
   COMPONENT clock_generator
      PORT( CLOCK_27     :IN  STD_LOGIC;
            reset        :IN  STD_LOGIC;
            AUD_XCK      :OUT STD_LOGIC);
   END COMPONENT;

   COMPONENT audio_and_video_config
      PORT( CLOCK_50,reset   :IN    STD_LOGIC;
            I2C_SDAT         :INOUT STD_LOGIC;
            I2C_SCLK         :OUT   STD_LOGIC);
   END COMPONENT;   

   COMPONENT audio_codec
      PORT( CLOCK_50,reset,read_s,write_s      :IN  STD_LOGIC;
            writedata_left, writedata_right    :IN  STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_ADCDAT,AUD_BCLK,AUD_ADCLRCK,
            AUD_DACLRCK                        :IN  STD_LOGIC;
            read_ready, write_ready            :OUT STD_LOGIC;
            readdata_left, readdata_right      :OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_DACDAT                         :OUT STD_LOGIC);
   END COMPONENT;
   
   COMPONENT noise_generator
      PORT( CLOCK_50    :IN  STD_LOGIC;
            read_s      :IN  STD_LOGIC;
            noise       :OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
   END COMPONENT;

   COMPONENT fifo_with_division
      GENERIC ( ELEMENT_COUNT : integer := 32);
      PORT( CLOCK_50, reset, read_s            :IN  STD_LOGIC;
            data_in                            :IN  STD_LOGIC_VECTOR(23 DOWNTO 0);
            current_data_out, last_data_out    :OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
   END COMPONENT;
   
   SIGNAL read_ready, write_ready, read_s, write_s      :STD_LOGIC;
   SIGNAL readdata_left, readdata_right                 :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL writedata_left, writedata_right               :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL reset                                         :STD_LOGIC;
   SIGNAL readdata_left_noisy, readdata_right_noisy     :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL noise                                         :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL buffer_left, buffer_right                     :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL divided_left, divided_right                   :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL result_left, result_right                     :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL sum_left, sum_right                           :STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL data_in                                       :STD_LOGIC_VECTOR(23 DOWNTO 0);
   
BEGIN
   reset <= NOT(KEY(0));

   read_s  <= write_ready AND read_ready;
   write_s <= write_ready AND read_ready;
   
   left_buffer: fifo_with_division
            GENERIC MAP ( ELEMENT_COUNT => 32)
            PORT MAP (CLOCK_50, reset, read_s, readdata_left+noise, divided_left, buffer_left);
   
   right_buffer: fifo_with_division
            GENERIC MAP ( ELEMENT_COUNT => 32)
            PORT MAP (CLOCK_50, reset, read_s, readdata_right+noise, divided_right, buffer_right);
   
   result_left  <= sum_left + divided_left - buffer_left;
   result_right <= sum_right + divided_right - buffer_right;
   
   PROCESS(CLOCK_50, reset, buffer_left, buffer_right, readdata_left_noisy, readdata_right_noisy)
      BEGIN
         IF (reset='0') THEN
            sum_left  <= "000000000000000000000000";
            sum_right <= "000000000000000000000000";
         ELSIF (RISING_EDGE(CLOCK_50)) THEN
            IF(read_s = '1' AND write_s = '1') THEN
               sum_left  <= result_left;
               sum_right <= result_right;
            END IF;
         END IF;         
   END PROCESS;

   writedata_left  <= result_left;
   writedata_right <= result_right;
   
   mynoise: noise_generator PORT MAP (CLOCK_50, read_s, noise);                  
   
   my_clock_gen: clock_generator PORT MAP (CLOCK2_50, reset, AUD_XCK);
   cfg: audio_and_video_config PORT MAP (CLOCK_50, reset, FPGA_I2C_SDAT, FPGA_I2C_SCLK);
   codec: audio_codec PORT MAP(CLOCK_50,reset,read_s,write_s,writedata_left, writedata_right,AUD_ADCDAT,AUD_BCLK,AUD_ADCLRCK,AUD_DACLRCK,read_ready, write_ready,readdata_left, readdata_right,AUD_DACDAT);

END Behavior;


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY fifo_with_division IS
   GENERIC ( ELEMENT_COUNT : integer := 32);
   PORT (CLOCK_50, reset, read_s              :IN  STD_LOGIC;
         data_in                              :IN  STD_LOGIC_VECTOR(23 DOWNTO 0);
         current_data_out, last_data_out      :OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
END fifo_with_division;


ARCHITECTURE Behavior OF fifo_with_division IS
   TYPE buffer_temp IS ARRAY(0 TO ELEMENT_COUNT-1) OF STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL buffer_s        :buffer_temp;
   SIGNAL input_divided   :STD_LOGIC_VECTOR(23 DOWNTO 0);
   
BEGIN
   -- This is the 24-bit input divided by 32. The 5 left-most bits are sign extension bits.
   input_divided <= ((data_in(23))&(data_in(23))&(data_in(23))&(data_in(23))&(data_in(23)))&(data_in(23 DOWNTO 5));
   
   -- Creation of successive fifo shift registers
   G2: FOR index IN 0 TO ELEMENT_COUNT-1 GENERATE
   
      PROCESS(CLOCK_50, reset, read_s, buffer_s, input_divided)
      BEGIN
         IF (reset='0') THEN
            buffer_s(index) <= "000000000000000000000000";
         ELSIF (RISING_EDGE(CLOCK_50)) THEN
            IF(read_s = '1') THEN
               IF(index = 0) THEN
                  buffer_s(index) <= input_divided;
               ELSE
                  buffer_s(index) <= buffer_s(index-1);
               END IF;
            END IF;
         END IF;   
      END PROCESS;

   END GENERATE;
   
   current_data_out <= input_divided;
   last_data_out    <= buffer_s(ELEMENT_COUNT-1);
   
END Behavior;

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY noise_generator IS
   PORT (CLOCK_50, read_s    :IN  STD_LOGIC;
         noise               :OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
END noise_generator;

ARCHITECTURE Behavior OF noise_generator IS
   SIGNAL counter            :STD_LOGIC_VECTOR(2 DOWNTO 0);
   SIGNAL Q                  :STD_LOGIC_VECTOR(9 DOWNTO 0);
BEGIN
   PROCESS(CLOCK_50)
   BEGIN
      IF ((CLOCK_50'EVENT) AND (CLOCK_50 = '1')) THEN
         IF (read_s = '1') THEN
            counter <= counter + '1';
         END IF;
      END IF;
   END PROCESS;
   
   Q <= (OTHERS => counter(2));
   noise <= Q&counter&"00000000000";
END Behavior;
