LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY part2 IS
   PORT ( CLOCK_50, CLOCK2_50, AUD_DACLRCK   : IN    STD_LOGIC;
          AUD_ADCLRCK, AUD_BCLK, AUD_ADCDAT  : IN    STD_LOGIC;
          KEY                                : IN    STD_LOGIC_VECTOR(0 DOWNTO 0);
          I2C_SDAT                      : INOUT STD_LOGIC;
          I2C_SCLK, AUD_DACDAT, AUD_XCK : OUT   STD_LOGIC );
END part2;

ARCHITECTURE Behavior OF part2 IS
   COMPONENT clock_generator
      PORT( CLOCK_27 :IN  STD_LOGIC;
            reset    :IN  STD_LOGIC;
            AUD_XCK  :OUT STD_LOGIC);
   END COMPONENT;

   COMPONENT audio_and_video_config
      PORT( CLOCK_50,reset : IN    STD_LOGIC;
            I2C_SDAT       : INOUT STD_LOGIC;
            I2C_SCLK       : OUT   STD_LOGIC);
   END COMPONENT;   

   COMPONENT audio_codec
      PORT( CLOCK_50, reset, read_s, write_s               : IN  STD_LOGIC;
            writedata_left, writedata_right                : IN  STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_ADCDAT, AUD_BCLK, AUD_ADCLRCK, AUD_DACLRCK : IN  STD_LOGIC;
            read_ready, write_ready                        : OUT STD_LOGIC;
            readdata_left, readdata_right                  : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_DACDAT                                     : OUT STD_LOGIC);
   END COMPONENT;
   
   COMPONENT noise_generator
      PORT( CLOCK_50 : IN  STD_LOGIC;
            read_s   : IN  STD_LOGIC;
            noise    : OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
   END COMPONENT;      

   SIGNAL reset : STD_LOGIC;
   SIGNAL read_ready, write_ready, read_s, write_s  : STD_LOGIC;
   SIGNAL readdata_left, readdata_right             : STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL writedata_left, writedata_right           : STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL WL0, WL1, WL2, WL3, WL4, WL5, WL6, WL_P   : STD_LOGIC_VECTOR(23 DOWNTO 0);   
   SIGNAL WR0, WR1, WR2, WR3, WR4, WR5, WR6, WR_P   : STD_LOGIC_VECTOR(23 DOWNTO 0);   
   SIGNAL readdata_left_noisy, readdata_right_noisy : STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL noise                                     : STD_LOGIC_VECTOR(23 DOWNTO 0);
   TYPE buf_l IS ARRAY(0 TO 6) OF STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL buffer_left : buf_l;
   TYPE buf_r IS ARRAY(0 TO 6) OF STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL buffer_right : buf_r;
BEGIN
   reset <= NOT(KEY(0));
   readdata_left_noisy  <= readdata_left + noise;
   readdata_right_noisy <= readdata_right + noise;

   read_s  <= write_ready AND read_ready;
   write_s <= write_ready AND read_ready;
   
   G1: FOR index IN 0 TO 5 GENERATE
      PROCESS(CLOCK_50, reset, buffer_left, buffer_right, readdata_left_noisy, 
				  readdata_right_noisy)
      BEGIN
         IF (reset='0') THEN
            buffer_left(index)  <= "000000000000000000000000";
            buffer_right(index) <= "000000000000000000000000";
         ELSIF (RISING_EDGE(CLOCK_50)) THEN
            IF(read_s = '1') THEN
               IF(index = 0) THEN
                  buffer_left(index)  <= readdata_left_noisy;
                  buffer_right(index) <= readdata_right_noisy;
               ELSE
                  buffer_left(index)  <= buffer_left(index-1);
                  buffer_right(index) <= buffer_right(index-1);
               END IF;
            END IF;
         END IF;   
      END PROCESS;
   END GENERATE;
   
   WL0 <= ((buffer_left(6)(23))&(buffer_left(6)(23))&(buffer_left(6)(23)))&
			 (buffer_left(6)(23 DOWNTO 3));
   WL1 <= ((buffer_left(5)(23))&(buffer_left(5)(23))&(buffer_left(5)(23)))&
			 (buffer_left(5)(23 DOWNTO 3)); 
   WL2 <= ((buffer_left(4)(23))&(buffer_left(4)(23))&(buffer_left(4)(23)))&
			 (buffer_left(4)(23 DOWNTO 3));
   WL3 <= ((buffer_left(3)(23))&(buffer_left(3)(23))&(buffer_left(3)(23)))&
			 (buffer_left(3)(23 DOWNTO 3));
   WL4 <= ((buffer_left(2)(23))&(buffer_left(2)(23))&(buffer_left(2)(23)))&
			 (buffer_left(2)(23 DOWNTO 3));
   WL5 <= ((buffer_left(1)(23))&(buffer_left(1)(23))&(buffer_left(1)(23)))&
			 (buffer_left(1)(23 DOWNTO 3)); 
   WL6 <= ((buffer_left(0)(23))&(buffer_left(0)(23))&(buffer_left(0)(23)))&
			 (buffer_left(0)(23 DOWNTO 3));
   WL_P <= ((readdata_left_noisy(23))&(readdata_left_noisy(23))&(readdata_left_noisy(23)))&
			  (readdata_left_noisy(23 DOWNTO 3));

   writedata_left <= WL0+WL1+WL2+WL3+WL4+WL5+WL6+WL_P;
               
   WR0 <= ((buffer_right(6)(23))&(buffer_right(6)(23))&(buffer_right(6)(23)))&
			 (buffer_right(6)(23 DOWNTO 3));
   WR1 <= ((buffer_right(5)(23))&(buffer_right(5)(23))&(buffer_right(5)(23)))&
			 (buffer_right(5)(23 DOWNTO 3)); 
   WR2 <= ((buffer_right(4)(23))&(buffer_right(4)(23))&(buffer_right(4)(23)))&
			 (buffer_right(4)(23 DOWNTO 3));
   WR3 <= ((buffer_right(3)(23))&(buffer_right(3)(23))&(buffer_right(3)(23)))&
			 (buffer_right(3)(23 DOWNTO 3));
   WR4 <= ((buffer_right(2)(23))&(buffer_right(2)(23))&(buffer_right(2)(23)))&
			 (buffer_right(2)(23 DOWNTO 3));
   WR5 <= ((buffer_right(1)(23))&(buffer_right(1)(23))&(buffer_right(1)(23)))&
			 (buffer_right(1)(23 DOWNTO 3)); 
   WR6 <= ((buffer_right(0)(23))&(buffer_right(0)(23))&(buffer_right(0)(23)))&
			 (buffer_right(0)(23 DOWNTO 3));
   WR_P <= ((readdata_right_noisy(23))&(readdata_right_noisy(23))&(readdata_right_noisy(23)))&
			  (readdata_right_noisy(23 DOWNTO 3));

   writedata_right <= WR0+WR1+WR2+WR3+WR4+WR5+WR6+WR_P;
   
   mynoise: noise_generator PORT MAP (CLOCK_50, read_s, noise);                  
   
   my_clock_gen: clock_generator PORT MAP (CLOCK2_50, reset, AUD_XCK);
   cfg: audio_and_video_config PORT MAP (CLOCK_50, reset, I2C_SDAT, I2C_SCLK);
   codec: audio_codec PORT MAP(CLOCK_50, reset, read_s, write_s, writedata_left, writedata_right,
	                            AUD_ADCDAT, AUD_BCLK, AUD_ADCLRCK, AUD_DACLRCK,read_ready, 
										 write_ready,readdata_left, readdata_right,AUD_DACDAT);
END Behavior;


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY noise_generator IS
   PORT (CLOCK_50, read_s      :IN STD_LOGIC;
         noise                 :OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
END noise_generator;

ARCHITECTURE Behavior OF noise_generator IS
   SIGNAL counter              :STD_LOGIC_VECTOR(2 DOWNTO 0);

BEGIN
   PROCESS(CLOCK_50)
   BEGIN
      IF (CLOCK_50'EVENT AND CLOCK_50 = '1') THEN
         IF (read_s = '1') THEN
            counter <= counter + '1';
         END IF;
      END IF;
   END PROCESS;
   
   noise <=(counter(2)&counter(2)&counter(2)&counter(2)&counter(2)&counter(2)&counter(2)&
			  counter(2)&counter(2)&counter(2))&counter&"00000000000";
END Behavior;
